module Cwallet
  module Wallet
    class NetHTTPClient < APIClient
      def initialize(base_uri, options = {})
        @conn = Net::HTTP.new(base_uri.host, base_uri.port)
        @conn.use_ssl = true if base_uri.scheme == 'https'
      end

      private

      def debug
        raise 'Need to be implemented'
      end

      def http_verb(method, path, body = nil, headers = {})
        case method
        when 'GET' then req = Net::HTTP::Get.new(path)
        when 'PUT' then req = Net::HTTP::Put.new(path)
        when 'POST' then req = Net::HTTP::Post.new(path)
        when 'DELETE' then req = Net::HTTP::Delete.new(path)
        else raise
        end

        req.body = body

        req['Content-Type'] = 'application/json'
        req['User-Agent'] = "cwallet/ruby/#{Cwallet::Wallet::VERSION}"
        auth_headers(method, path, body).each do |key, val|
          req[key] = val
        end
        headers.each do |key, val|
          req[key] = val
        end

        if debug
          $stdout.puts("=====REQUEST:\n#{req.method} #{req.path}\nHEADERS: #{req.to_hash}\nBODY: #{req.body}\n=====")
        end

        resp = @conn.request(req)
        out = NetHTTPResponse.new(resp)

        if debug
          $stdout.puts("=====RESPONSE:\nSTATUS: #{out.status}\nBODY: #{out.body}\n=====")
        end

        Cwallet::Wallet::check_response_status(out)
        yield(out)
        out.data
      end
    end

    # Net-Http response object
    class NetHTTPResponse < APIResponse
      def body
        JSON.parse(@response.body) rescue {}
      end

      def body=(body)
        @response.body = body.to_json
      end

      def data
        body['data']
      end

      def headers
        out = @response.to_hash.map do |key, val|
          [ key.upcase.gsub('_', '-'), val.count == 1 ? val.first : val ]
        end
        out.to_h
      end

      def status
        @response.code.to_i
      end
    end
  end
end

